import * as v from 'valibot';

const commonInputSchema = v.object({});

export type CommonInput = v.Output<typeof commonInputSchema>;

export const BOOK_FILENAME = 'patchouli.book.json' as const;

export const BINDING_FILENAME = 'patchouli.binding.json' as const;

export const bindingFileBaseSchema = v.object({
	generatorType: v.string(),
	generatorConfigFilePath: v.optional(v.string()),
	generatorDistDirectory: v.string(),
	pageFileExtensions: v.array(v.string()),
	pageOrdering: v.optional(
		v.array(
			v.string()
		)
	), // allow blobs on page ordering // ahh so that's why I added this comment, so we can add the generated files in binding
});

export const bookFileSchema = v.object({
	title: v.string(),
	publisher: v.optional(v.string()),
	isbn: v.optional(v.string()),
	id: v.optional(v.string()),
	creator: v.string(),
	contributors: v.optional(v.array(v.string())),
	description: v.optional(v.string()),
	subjects: v.optional(v.array(v.string())),
	rights: v.optional(v.string()),
});

export type Book = v.Output<typeof bookFileSchema>;

export type Binding = v.Output<typeof bindingFileBaseSchema>;

export interface BookManifestItem {
	id: string;
	href: string;
	path?: string;
	buffer?: Buffer;
	type?: string;
	properties?: string;
}

export interface BookData {
	metadata: Book;
	manifest: BookManifestItem[];
	spine: BookManifestItem['id'][];
	binding: Binding;
}
