import {js2xml} from 'xml-js';
import {BookData} from '../../common';

export const prepareEpubContainerContents = (rootFilePath: string) => {
	return js2xml({
		_declaration: {
			_attributes: {
				version: '1.0',
				encoding: 'utf-8',
			},
		},
		container: {
			_attributes: {
				xmlns: 'urn:oasis:names:tc:opendocument:xmlns:container',
				version: '1.0',
			},
			rootfiles: {
				rootfile: {
					_attributes: {
						'full-path': rootFilePath,
						'media-type': 'application/oebps-package+xml',
					}
				}
			}
		}
	}, {
		compact: true,
	});
};

export const prepareEpubRootfileContents = async <U extends BookData = BookData>(bookPackage: U) => {
	const { default: mime } = await import('mime'); // ESM import, convert to CJS for compatibility
	const uniqueIdentifierName = 'BookID';
	return js2xml({
		declaration: {
			attributes: {
				version: '1.0',
				encoding: 'utf-8',
			},
		},
		elements: [
			{
				type: 'element',
				name: 'package',
				attributes: {
					xmlns: 'http://www.idpf.org/2007/opf',
					'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
					version: '3.0',
					'unique-identifier': uniqueIdentifierName,
				},
				elements: [
					{
						type: 'element',
						name: 'metadata',
						elements: [
							{
								type: 'element',
								name: 'dc:identifier',
								attributes: {
									id: uniqueIdentifierName,
								},
								elements: [
									{
										type: 'text',
										text: bookPackage.metadata.id,
									},
								],
							},
							typeof bookPackage.metadata.isbn !== 'undefined'
								? {
									type: 'element',
									name: 'dc:identifier',
									elements: [
										{
											type: 'text',
											text: bookPackage.metadata.isbn,
										},
									],
								}
								: undefined,
							{
								type: 'element',
								name: 'dc:title',
								elements: [
									{
										type: 'text',
										text: bookPackage.metadata.title,
									},
								],
							},
							typeof bookPackage.metadata.publisher !== 'undefined'
								? {
									type: 'element',
									name: 'dc:publisher',
									elements: [
										{
											type: 'text',
											text: bookPackage.metadata.publisher,
										},
									],
								}
								: undefined,
							{
								type: 'element',
								name: 'dc:creator',
								elements: [
									{
										type: 'text',
										text: bookPackage.metadata.creator,
									},
								],
							},
							...(bookPackage.metadata.contributors ?? []).map((c) => ({
								type: 'element',
								name: 'dc:contributor',
								elements: [
									{
										type: 'text',
										text: c,
									},
								],
							})),
							typeof bookPackage.metadata.description !== 'undefined'
								? {
									type: 'element',
									name: 'dc:description',
									elements: [
										{
											type: 'text',
											text: bookPackage.metadata.description,
										},
									],
								}
								: undefined,
							...(bookPackage.metadata.subjects ?? []).map((c) => ({
								type: 'element',
								name: 'dc:subject',
								elements: [
									{
										type: 'text',
										text: c,
									},
								],
							})),
							typeof bookPackage.metadata.rights !== 'undefined'
								? {
									type: 'element',
									name: 'dc:rights',
									elements: [
										{
											type: 'text',
											text: bookPackage.metadata.rights,
										},
									],
								}
								: undefined,
						]
							.filter((s) => typeof s !== 'undefined'),
					},
					{
						type: 'element',
						name: 'manifest',
						elements: bookPackage.manifest.map((e) => ({
							type: 'element',
							name: 'item',
							attributes: {
								id: e.id,
								href: e.href,
								'media-type': mime.getType(e.href),
							},
						})),
					},
					{
						type: 'element',
						name: 'spine',
						elements: bookPackage.spine.map((e) => ({
							type: 'element',
							name: 'itemref',
							attributes: {
								idref: bookPackage.manifest.find((m) => m.href === e || m.path === e)?.id,
							},
						})),
					},
				],
			},
		],
	}, {
		compact: false,
	});
};
