import {PDFDocument} from 'pdf-lib';
import {Book} from '../../common';

export const compilePdfBuffers = async (buffers: Buffer[]) => {
	const documents = await buffers.reduce(
		async (previousPromise, e) => {
			const p = await previousPromise;
			const d = await PDFDocument.load(e, {
				updateMetadata: false,
			});
			return [
				...p,
				d,
			];
		},
		Promise.resolve([] as PDFDocument[])
	);
	return await documents.reduce(
		async (thePdfDocPromise, dd) => {
			const thePdfDoc = await thePdfDocPromise;
			const c = await thePdfDoc.copyPages(dd, dd.getPageIndices());
			c.forEach((page) => {
				thePdfDoc.addPage(page);
			});
			return thePdfDoc;
		},
		PDFDocument.create(),
	);
};

export const addPdfViewerPreferences = (pdfDoc: PDFDocument) => {
	const viewerPrefs = pdfDoc.catalog.getOrCreateViewerPreferences();
	viewerPrefs.setHideToolbar(false);
	viewerPrefs.setHideMenubar(false);
	viewerPrefs.setDisplayDocTitle(true);
	return pdfDoc;
};

export const addPdfMetadata = (pdfDoc: PDFDocument, metadata: Book) => {
	pdfDoc.setTitle(metadata.title, { showInWindowTitleBar: true });
	pdfDoc.setAuthor([metadata.creator, ...(metadata.contributors ?? [])].join(', '));
	if (Array.isArray(metadata.subjects)) {
		pdfDoc.setSubject(metadata.subjects.join(', '));
	}
	pdfDoc.setCreator('@modal-sh/patchouli');
	const defaultProducer = pdfDoc.getProducer();
	pdfDoc.setProducer(['puppeteer-html-pdf (https://github.com/ultimateakash/puppeteer-html-pdf)', defaultProducer].join(', '));
	const now = new Date();
	pdfDoc.setCreationDate(now);
	pdfDoc.setModificationDate(now);
	// TODO add language data
	return pdfDoc;
};
