import {BookData} from '../../common';
import {renderFilesAsPdf} from './rendering';
import {addPdfMetadata, addPdfViewerPreferences, compilePdfBuffers} from './compiling';

export const name = 'pdf' as const;

export const bindBook = async <T extends BookData = BookData>(input: T) => {
	const buffers = await renderFilesAsPdf(input.spine);
	// TODO how to generate TOC?
	//   https://github.com/Hopding/pdf-lib/issues/123
	//   https://github.com/Hopding/pdf-lib/issues/1257
	const compiledDocument = await compilePdfBuffers(buffers);
	const documentWithMetadata = addPdfMetadata(compiledDocument, input.metadata);
	const documentWithViewerPreferences = addPdfViewerPreferences(documentWithMetadata);;
	const buffer = await documentWithViewerPreferences.save();
	return Buffer.from(buffer);
};
