import * as v from 'valibot';
import assert from 'assert';
import * as PdfFormat from './formats/pdf';
import * as EpubFormat from './formats/epub';
import * as DirCompiler from './compilers/dir';
import * as ArchiveCompiler from './compilers/archive';

const AVAILABLE_COMPILERS = [
	ArchiveCompiler,
	DirCompiler,
];

const AVAILABLE_FORMATS = [
	PdfFormat,
	EpubFormat
];

const optionsSchema = v.object(
	{
		input: v.union(AVAILABLE_COMPILERS.map((c) => c.inputSchema)),
		output: v.object(
			{
				format: v.picklist(AVAILABLE_FORMATS.map((f) => f.name)),
				path: v.string(),
			},
			v.unknown(),
		),
	},
	v.never(),
)

export interface BindFunctionOptions extends v.Output<typeof optionsSchema> {}

export interface BindFunction {
	<T extends BindFunctionOptions = BindFunctionOptions>(options: T): Promise<Buffer>;
}

// TODO add options to include blank pages to ensure chapters' first pages are in the right side of the book
export const bindBook: BindFunction = async (options: BindFunctionOptions): Promise<Buffer> => {
	const { input, output, } = await v.parseAsync(optionsSchema, options);
	const selectedCompiler = AVAILABLE_COMPILERS.find((c) => c.name === input.sourceType);
	assert(typeof selectedCompiler !== 'undefined');
	const bookPackage = await selectedCompiler.compileFromInput(input);
	const selectedFormat = AVAILABLE_FORMATS.find((f) => f.name === output.format);
	assert(typeof selectedFormat !== 'undefined');
	return selectedFormat.bindBook(bookPackage);
};
